@@ -25,6 +25,8 @@ const QString CustomData::Created = QStringLiteral("_CREATED");
 const QString CustomData::BrowserKeyPrefix = QStringLiteral("KPXC_BROWSER_");
 const QString CustomData::BrowserLegacyKeyPrefix = QStringLiteral("Public Key: ");
 const QString CustomData::ExcludeFromReportsLegacy = QStringLiteral("KnownBad");
+const QString CustomData::FdoSecretsExposedGroup = QStringLiteral("FDO_SECRETS_EXPOSED_GROUP");
+const QString CustomData::RandomSlug = QStringLiteral("KPXC_RANDOM_SLUG");
 
 // Fallback item for return by reference
 static const CustomData::CustomDataItem NULL_ITEM{};
@@ -188,7 +190,12 @@ void CustomData::updateLastModified(QDateTime lastModified)
 
 bool CustomData::isProtected(const QString& key) const
 {
-    return key.startsWith(CustomData::BrowserKeyPrefix) || key.startsWith(CustomData::Created);
+    return key.startsWith(BrowserKeyPrefix) || key == Created || key == FdoSecretsExposedGroup;
+}
+
+bool CustomData::isAutoGenerated(const QString& key) const
+{
+    return key == LastModified || key == RandomSlug;
 }
 
 bool CustomData::operator==(const CustomData& other) const
