@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2023 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2024 KeePassXC Team <team@keepassxc.org>
  *  Copyright (C) 2010 Felix Geyer <debfx@fobos.de>
  *
  *  This program is free software: you can redistribute it and/or modify
@@ -120,6 +120,9 @@ public:
     bool excludeFromReports() const;
     void setExcludeFromReports(bool state);
 
+    bool hasPasskey() const;
+    void removePasskey();
+
     bool hasTotp() const;
     bool isExpired() const;
     bool willExpireInDays(int days) const;
@@ -224,9 +227,6 @@ public:
     };
 
     static const int DefaultIconNumber;
-    static const int ResolveMaximumDepth;
-    static const QString AutoTypeSequenceUsername;
-    static const QString AutoTypeSequencePassword;
 
     /**
      * Creates a duplicate of this entry except that the returned entry isn't
