@@ -21,15 +21,6 @@
 #include "core/Database.h"
 #include "core/Metadata.h"
 
-namespace Keys
-{
-    namespace Db
-    {
-        constexpr auto FdoSecretsExposedGroup = "FDO_SECRETS_EXPOSED_GROUP";
-    } // namespace Db
-
-} // namespace Keys
-
 namespace FdoSecrets
 {
 
@@ -98,20 +89,19 @@ namespace FdoSecrets
         return exposedGroup(db.data());
     }
 
-    void FdoSecretsSettings::setExposedGroup(const QSharedPointer<Database>& db,
-                                             const QUuid& group) // clazy:exclude=function-args-by-value
+    void FdoSecretsSettings::setExposedGroup(const QSharedPointer<Database>& db, const QUuid& group)
     {
         setExposedGroup(db.data(), group);
     }
 
     QUuid FdoSecretsSettings::exposedGroup(Database* db) const
     {
-        return {db->metadata()->customData()->value(Keys::Db::FdoSecretsExposedGroup)};
+        return {db->metadata()->customData()->value(CustomData::FdoSecretsExposedGroup)};
     }
 
-    void FdoSecretsSettings::setExposedGroup(Database* db, const QUuid& group) // clazy:exclude=function-args-by-value
+    void FdoSecretsSettings::setExposedGroup(Database* db, const QUuid& group)
     {
-        db->metadata()->customData()->set(Keys::Db::FdoSecretsExposedGroup, group.toString());
+        db->metadata()->customData()->set(CustomData::FdoSecretsExposedGroup, group.toString());
     }
 
 } // namespace FdoSecrets
