@@ -19,13 +19,15 @@
 #include "DatabaseOpenWidget.h"
 #include "ui_DatabaseOpenWidget.h"
 
-#include "config-keepassx.h"
 #include "gui/FileDialog.h"
 #include "gui/Icons.h"
 #include "gui/MainWindow.h"
 #include "gui/MessageBox.h"
 #include "keys/ChallengeResponseKey.h"
 #include "keys/FileKey.h"
+#ifdef WITH_XC_YUBIKEY
+#include "keys/drivers/YubiKeyInterfaceUSB.h"
+#endif
 
 #ifdef Q_OS_MACOS
 #include "touchid/TouchID.h"
@@ -54,25 +56,15 @@ namespace
         }
         return false;
     }
-
-    bool canPerformQuickUnlock(const QString& filename)
-    {
-        if (isQuickUnlockAvailable()) {
-#if defined(Q_CC_MSVC)
-            return getWindowsHello()->hasKey(filename);
-#elif defined(Q_OS_MACOS)
-            return TouchID::getInstance().containsKey(filename);
-#endif
-        }
-        Q_UNUSED(filename);
-        return false;
-    }
 } // namespace
 
 DatabaseOpenWidget::DatabaseOpenWidget(QWidget* parent)
     : DialogyWidget(parent)
     , m_ui(new Ui::DatabaseOpenWidget())
     , m_db(nullptr)
+#ifdef WITH_XC_YUBIKEY
+    , m_deviceListener(new DeviceListener(this))
+#endif
 {
     m_ui->setupUi(this);
 
@@ -105,18 +97,23 @@ DatabaseOpenWidget::DatabaseOpenWidget(QWidget* parent)
     connect(m_ui->buttonBox, SIGNAL(accepted()), SLOT(openDatabase()));
     connect(m_ui->buttonBox, SIGNAL(rejected()), SLOT(reject()));
 
-    m_ui->hardwareKeyLabelHelp->setIcon(icons()->icon("system-help").pixmap(QSize(12, 12)));
-    connect(m_ui->hardwareKeyLabelHelp, SIGNAL(clicked(bool)), SLOT(openHardwareKeyHelp()));
-    m_ui->keyFileLabelHelp->setIcon(icons()->icon("system-help").pixmap(QSize(12, 12)));
-    connect(m_ui->keyFileLabelHelp, SIGNAL(clicked(bool)), SLOT(openKeyFileHelp()));
+    connect(m_ui->addKeyFileLinkLabel, &QLabel::linkActivated, this, &DatabaseOpenWidget::browseKeyFile);
+    connect(m_ui->keyFileLineEdit, &PasswordWidget::textChanged, this, [&](const QString& text) {
+        bool state = !text.isEmpty();
+        m_ui->addKeyFileLinkLabel->setVisible(!state);
+        m_ui->selectKeyFileComponent->setVisible(state);
+    });
+    connect(m_ui->useHardwareKeyCheckBox, &QCheckBox::toggled, m_ui->hardwareKeyCombo, &QComboBox::setEnabled);
+
+    m_ui->selectKeyFileComponent->setVisible(false);
+    toggleHardwareKeyComponent(false);
 
-#ifdef WITH_XC_YUBIKEY
-    m_ui->hardwareKeyProgress->setVisible(false);
     QSizePolicy sp = m_ui->hardwareKeyProgress->sizePolicy();
     sp.setRetainSizeWhenHidden(true);
     m_ui->hardwareKeyProgress->setSizePolicy(sp);
 
-    connect(m_ui->buttonRedetectYubikey, SIGNAL(clicked()), SLOT(pollHardwareKey()));
+#ifdef WITH_XC_YUBIKEY
+    connect(m_deviceListener, SIGNAL(devicePlugged(bool, void*, void*)), this, SLOT(pollHardwareKey()));
     connect(YubiKey::instance(), SIGNAL(detectComplete(bool)), SLOT(hardwareKeyResponse(bool)), Qt::QueuedConnection);
 
     connect(YubiKey::instance(), &YubiKey::userInteractionRequest, this, [this] {
@@ -128,12 +125,17 @@ DatabaseOpenWidget::DatabaseOpenWidget(QWidget* parent)
         }
     });
     connect(YubiKey::instance(), &YubiKey::challengeCompleted, this, [this] { m_ui->messageWidget->hide(); });
+
+    m_ui->noHardwareKeysFoundLabel->setVisible(false);
+    m_ui->refreshHardwareKeys->setIcon(icons()->icon("yubikey-refresh", true));
+    connect(m_ui->refreshHardwareKeys, &QPushButton::clicked, this, [this] { pollHardwareKey(true); });
+    m_hideNoHardwareKeysFoundTimer.setInterval(2000);
+    connect(&m_hideNoHardwareKeysFoundTimer, &QTimer::timeout, this, [this] {
+        m_ui->noHardwareKeysFoundLabel->setVisible(false);
+    });
 #else
-    m_ui->hardwareKeyLabel->setVisible(false);
-    m_ui->hardwareKeyLabelHelp->setVisible(false);
-    m_ui->buttonRedetectYubikey->setVisible(false);
-    m_ui->challengeResponseCombo->setVisible(false);
-    m_ui->hardwareKeyProgress->setVisible(false);
+    m_ui->noHardwareKeysFoundLabel->setVisible(false);
+    m_ui->refreshHardwareKeys->setVisible(false);
 #endif
 
     // QuickUnlock actions
@@ -146,28 +148,79 @@ DatabaseOpenWidget::~DatabaseOpenWidget()
 {
 }
 
-void DatabaseOpenWidget::showEvent(QShowEvent* event)
+void DatabaseOpenWidget::toggleHardwareKeyComponent(bool state)
 {
-    DialogyWidget::showEvent(event);
-    if (isOnQuickUnlockScreen()) {
-        m_ui->quickUnlockButton->setFocus();
-        if (!canPerformQuickUnlock(m_filename)) {
-            resetQuickUnlock();
-        }
+    m_ui->hardwareKeyProgress->setVisible(false);
+    m_ui->hardwareKeyComponent->setVisible(state);
+    m_ui->hardwareKeyCombo->setVisible(state && m_ui->hardwareKeyCombo->count() != 1);
+    m_ui->noHardwareKeysFoundLabel->setVisible(!state && m_manualHardwareKeyRefresh);
+    if (!state) {
+        m_ui->useHardwareKeyCheckBox->setChecked(false);
+    }
+    if (m_ui->hardwareKeyCombo->count() == 1) {
+        m_ui->useHardwareKeyCheckBox->setText(
+            tr("Use hardware key [Serial: %1]")
+                .arg(m_ui->hardwareKeyCombo->itemData(m_ui->hardwareKeyCombo->currentIndex())
+                         .value<YubiKeySlot>()
+                         .first));
     } else {
-        m_ui->editPassword->setFocus();
+        m_ui->useHardwareKeyCheckBox->setText(tr("Use hardware key"));
     }
-    m_hideTimer.stop();
 }
 
-void DatabaseOpenWidget::hideEvent(QHideEvent* event)
+bool DatabaseOpenWidget::event(QEvent* event)
 {
-    DialogyWidget::hideEvent(event);
+    bool ret = DialogyWidget::event(event);
+    auto type = event->type();
 
-    // Schedule form clearing if we are hidden
-    if (!isVisible()) {
-        m_hideTimer.start();
+    if (type == QEvent::Show || type == QEvent::WindowActivate) {
+        if (isOnQuickUnlockScreen() && (m_db.isNull() || !canPerformQuickUnlock())) {
+            resetQuickUnlock();
+        }
+        toggleQuickUnlockScreen();
+
+        if (type == QEvent::Show) {
+#ifdef WITH_XC_YUBIKEY
+#ifdef Q_OS_WIN
+            m_deviceListener->registerHotplugCallback(true,
+                                                      true,
+                                                      YubiKeyInterfaceUSB::YUBICO_USB_VID,
+                                                      DeviceListener::MATCH_ANY,
+                                                      &DeviceListenerWin::DEV_CLS_KEYBOARD);
+            m_deviceListener->registerHotplugCallback(true,
+                                                      true,
+                                                      YubiKeyInterfaceUSB::ONLYKEY_USB_VID,
+                                                      DeviceListener::MATCH_ANY,
+                                                      &DeviceListenerWin::DEV_CLS_KEYBOARD);
+#else
+            m_deviceListener->registerHotplugCallback(true, true, YubiKeyInterfaceUSB::YUBICO_USB_VID);
+            m_deviceListener->registerHotplugCallback(true, true, YubiKeyInterfaceUSB::ONLYKEY_USB_VID);
+#endif
+#endif
+        }
+
+        if (isVisible()) {
+            m_hideTimer.stop();
+            pollHardwareKey();
+        }
+
+        ret = true;
+    } else if (type == QEvent::Hide || type == QEvent::WindowDeactivate) {
+        // Schedule form clearing if we are hidden
+        if (!m_hideTimer.isActive()) {
+            m_hideTimer.start();
+        }
+
+#ifdef WITH_XC_YUBIKEY
+        if (type == QEvent::Hide) {
+            m_deviceListener->deregisterAllHotplugCallbacks();
+        }
+#endif
+
+        ret = true;
     }
+
+    return ret;
 }
 
 bool DatabaseOpenWidget::unlockingDatabase()
@@ -189,21 +242,11 @@ void DatabaseOpenWidget::load(const QString& filename)
         }
     }
 
-    if (canPerformQuickUnlock(m_filename)) {
-        m_ui->centralStack->setCurrentIndex(1);
-        m_ui->quickUnlockButton->setFocus();
-    } else {
-        m_ui->editPassword->setFocus();
-    }
+    toggleQuickUnlockScreen();
 
 #ifdef WITH_XC_YUBIKEY
-    // Only auto-poll for hardware keys if we previously used one with this database file
-    if (config()->get(Config::RememberLastKeyFiles).toBool()) {
-        auto lastChallengeResponse = config()->get(Config::LastChallengeResponse).toHash();
-        if (lastChallengeResponse.contains(m_filename)) {
-            pollHardwareKey();
-        }
-    }
+    // Do initial auto-poll
+    pollHardwareKey();
 #endif
 }
 
@@ -215,9 +258,10 @@ void DatabaseOpenWidget::clearForms()
     m_ui->keyFileLineEdit->clear();
     m_ui->keyFileLineEdit->setShowPassword(false);
     m_ui->keyFileLineEdit->setClearButtonEnabled(true);
-    m_ui->challengeResponseCombo->clear();
-    m_ui->centralStack->setCurrentIndex(0);
-    m_db.reset();
+    m_ui->hardwareKeyCombo->clear();
+    toggleQuickUnlockScreen();
+
+    m_db.reset(new Database(m_filename));
 }
 
 QSharedPointer<Database> DatabaseOpenWidget::database()
@@ -273,6 +317,7 @@ void DatabaseOpenWidget::openDatabase()
             auto btn = msgBox->addButton(tr("Open database anyway"), QMessageBox::ButtonRole::AcceptRole);
             msgBox->setDefaultButton(btn);
             msgBox->addButton(QMessageBox::Cancel);
+            msgBox->layout()->setSizeConstraint(QLayout::SetMinimumSize);
             msgBox->exec();
             if (msgBox->clickedButton() != btn) {
                 m_db.reset(new Database());
@@ -340,7 +385,7 @@ QSharedPointer<CompositeKey> DatabaseOpenWidget::buildDatabaseKey()
 {
     auto databaseKey = QSharedPointer<CompositeKey>::create();
 
-    if (canPerformQuickUnlock(m_filename)) {
+    if (!m_db.isNull() && canPerformQuickUnlock()) {
         // try to retrieve the stored password using Windows Hello
         QByteArray keyData;
 #ifdef Q_CC_MSVC
@@ -411,9 +456,9 @@ QSharedPointer<CompositeKey> DatabaseOpenWidget::buildDatabaseKey()
     auto lastChallengeResponse = config()->get(Config::LastChallengeResponse).toHash();
     lastChallengeResponse.remove(m_filename);
 
-    int selectionIndex = m_ui->challengeResponseCombo->currentIndex();
-    if (selectionIndex > 0) {
-        auto slot = m_ui->challengeResponseCombo->itemData(selectionIndex).value<YubiKeySlot>();
+    int selectionIndex = m_ui->hardwareKeyCombo->currentIndex();
+    if (m_ui->useHardwareKeyCheckBox->isChecked()) {
+        auto slot = m_ui->hardwareKeyCombo->itemData(selectionIndex).value<YubiKeySlot>();
         auto crKey = QSharedPointer<ChallengeResponseKey>(new ChallengeResponseKey(slot));
         databaseKey->addChallengeResponseKey(crKey);
 
@@ -434,55 +479,72 @@ void DatabaseOpenWidget::reject()
     emit dialogFinished(false);
 }
 
-void DatabaseOpenWidget::browseKeyFile()
+bool DatabaseOpenWidget::browseKeyFile()
 {
     QString filters = QString("%1 (*);;%2 (*.keyx; *.key)").arg(tr("All files"), tr("Key files"));
-    QString filename = fileDialog()->getOpenFileName(this, tr("Select key file"), QString(), filters);
+    QString filename =
+        fileDialog()->getOpenFileName(this, tr("Select key file"), FileDialog::getLastDir("keyfile"), filters);
+    if (filename.isEmpty()) {
+        return false;
+    }
+    if (config()->get(Config::RememberLastKeyFiles).toBool()) {
+        FileDialog::saveLastDir("keyfile", filename, true);
+    } else {
+        FileDialog::saveLastDir("keyfile", {});
+    }
 
     if (QFileInfo(filename).canonicalFilePath() == QFileInfo(m_filename).canonicalFilePath()) {
         MessageBox::warning(this,
                             tr("Cannot use database file as key file"),
-                            tr("You cannot use your database file as a key file.\nIf you do not have a key file, "
-                               "please leave the field empty."),
+                            tr("Your database file is NOT a key file!\nIf you don't have a key file or don't know what "
+                               "that is, you don't have to select one."),
                             MessageBox::Button::Ok);
-        filename = "";
+        return false;
+    }
+    if (filename.endsWith(".kdbx")
+        && MessageBox::warning(this,
+                               tr("KeePassXC database file selected"),
+                               tr("The file you selected looks like a database file.\nA database file is NOT a key "
+                                  "file!\n\nAre you sure you want to continue with this file?."),
+                               MessageBox::Button::Yes | MessageBox::Button::Cancel,
+                               MessageBox::Button::Cancel)
+               != MessageBox::Yes) {
+        return false;
     }
 
-    if (!filename.isEmpty()) {
-        m_ui->keyFileLineEdit->setText(filename);
-    }
+    m_ui->keyFileLineEdit->setText(filename);
+    return true;
 }
 
-void DatabaseOpenWidget::pollHardwareKey()
+void DatabaseOpenWidget::pollHardwareKey(bool manualTrigger)
 {
     if (m_pollingHardwareKey) {
         return;
     }
 
-    m_ui->challengeResponseCombo->clear();
-    m_ui->challengeResponseCombo->addItem(tr("Detecting hardware keys…"));
-
-    m_ui->buttonRedetectYubikey->setEnabled(false);
-    m_ui->challengeResponseCombo->setEnabled(false);
+    m_ui->hardwareKeyCombo->setEnabled(false);
     m_ui->hardwareKeyProgress->setVisible(true);
+    m_ui->refreshHardwareKeys->setEnabled(false);
+    m_ui->noHardwareKeysFoundLabel->setVisible(false);
     m_pollingHardwareKey = true;
+    m_manualHardwareKeyRefresh = manualTrigger;
 
-    YubiKey::instance()->findValidKeysAsync();
+    // Add a delay, if this is an automatic trigger, to allow the USB device to settle as
+    // the device may not report a valid serial number immediately after plugging in
+    int delay = manualTrigger ? 0 : 500;
+    QTimer::singleShot(delay, this, [] { YubiKey::instance()->findValidKeysAsync(); });
 }
 
 void DatabaseOpenWidget::hardwareKeyResponse(bool found)
 {
-    m_ui->challengeResponseCombo->clear();
-    m_ui->buttonRedetectYubikey->setEnabled(true);
     m_ui->hardwareKeyProgress->setVisible(false);
+    m_ui->refreshHardwareKeys->setEnabled(true);
+    m_ui->hardwareKeyCombo->clear();
     m_pollingHardwareKey = false;
 
     if (!found) {
-        m_ui->challengeResponseCombo->addItem(tr("No hardware keys detected"));
-        m_ui->challengeResponseCombo->setEnabled(false);
+        toggleHardwareKeyComponent(false);
         return;
-    } else {
-        m_ui->challengeResponseCombo->addItem(tr("Select hardware key…"));
     }
 
     YubiKeySlot lastUsedSlot;
@@ -494,31 +556,24 @@ void DatabaseOpenWidget::hardwareKeyResponse(bool found)
             if (split.size() > 1) {
                 lastUsedSlot = YubiKeySlot(split[0].toUInt(), split[1].toInt());
             }
+            m_ui->useHardwareKeyCheckBox->setChecked(true);
         }
     }
 
     int selectedIndex = 0;
-    for (auto& slot : YubiKey::instance()->foundKeys()) {
+    const auto foundKeys = YubiKey::instance()->foundKeys();
+    for (auto i = foundKeys.cbegin(); i != foundKeys.cend(); ++i) {
         // add detected YubiKey to combo box
-        m_ui->challengeResponseCombo->addItem(YubiKey::instance()->getDisplayName(slot), QVariant::fromValue(slot));
+        m_ui->hardwareKeyCombo->addItem(i.value(), QVariant::fromValue(i.key()));
         // Select this YubiKey + Slot if we used it in the past
-        if (lastUsedSlot == slot) {
-            selectedIndex = m_ui->challengeResponseCombo->count() - 1;
+        if (lastUsedSlot == i.key()) {
+            selectedIndex = m_ui->hardwareKeyCombo->count() - 1;
         }
     }
 
-    m_ui->challengeResponseCombo->setCurrentIndex(selectedIndex);
-    m_ui->challengeResponseCombo->setEnabled(true);
-}
-
-void DatabaseOpenWidget::openHardwareKeyHelp()
-{
-    QDesktopServices::openUrl(QUrl("https://keepassxc.org/docs#faq-cat-yubikey"));
-}
-
-void DatabaseOpenWidget::openKeyFileHelp()
-{
-    QDesktopServices::openUrl(QUrl("https://keepassxc.org/docs#faq-cat-keyfile"));
+    toggleHardwareKeyComponent(true);
+    m_ui->hardwareKeyCombo->setEnabled(m_ui->useHardwareKeyCheckBox->isChecked());
+    m_ui->hardwareKeyCombo->setCurrentIndex(selectedIndex);
 }
 
 void DatabaseOpenWidget::setUserInteractionLock(bool state)
@@ -536,11 +591,40 @@ void DatabaseOpenWidget::setUserInteractionLock(bool state)
     m_unlockingDatabase = state;
 }
 
-bool DatabaseOpenWidget::isOnQuickUnlockScreen()
+bool DatabaseOpenWidget::canPerformQuickUnlock() const
+{
+    if (!m_db.isNull() && isQuickUnlockAvailable()) {
+#if defined(Q_CC_MSVC)
+        return getWindowsHello()->hasKey(m_filename);
+#elif defined(Q_OS_MACOS)
+        return TouchID::getInstance().containsKey(m_filename);
+#endif
+    }
+    return false;
+}
+
+bool DatabaseOpenWidget::isOnQuickUnlockScreen() const
 {
     return m_ui->centralStack->currentIndex() == 1;
 }
 
+void DatabaseOpenWidget::toggleQuickUnlockScreen()
+{
+    if (canPerformQuickUnlock()) {
+        m_ui->centralStack->setCurrentIndex(1);
+        // Work around qt issue where focus is stolen even if not visible
+        if (m_ui->quickUnlockButton->isVisible()) {
+            m_ui->quickUnlockButton->setFocus();
+        }
+    } else {
+        m_ui->centralStack->setCurrentIndex(0);
+        // Work around qt issue where focus is stolen even if not visible
+        if (m_ui->editPassword->isVisible()) {
+            m_ui->editPassword->setFocus();
+        }
+    }
+}
+
 void DatabaseOpenWidget::triggerQuickUnlock()
 {
     if (isOnQuickUnlockScreen()) {
