@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2024 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <QTabBar>
 
 #include "autotype/AutoType.h"
+#include "core/Merger.h"
 #include "core/Tools.h"
 #include "format/CsvExporter.h"
 #include "gui/Clipboard.h"
@@ -28,19 +29,20 @@
 #include "gui/DatabaseWidget.h"
 #include "gui/DatabaseWidgetStateSync.h"
 #include "gui/FileDialog.h"
-#include "gui/HtmlExporter.h"
 #include "gui/MessageBox.h"
 #include "gui/export/ExportDialog.h"
 #ifdef Q_OS_MACOS
 #include "gui/osutils/macutils/MacUtils.h"
 #endif
 #include "gui/wizard/NewDatabaseWizard.h"
+#include "wizard/ImportWizard.h"
 
 DatabaseTabWidget::DatabaseTabWidget(QWidget* parent)
     : QTabWidget(parent)
     , m_dbWidgetStateSync(new DatabaseWidgetStateSync(this))
     , m_dbWidgetPendingLock(nullptr)
     , m_databaseOpenDialog(new DatabaseOpenDialog(this))
+    , m_databaseOpenInProgress(false)
 {
     auto* tabBar = new QTabBar(this);
     tabBar->setAcceptDrops(true);
@@ -55,7 +57,7 @@ DatabaseTabWidget::DatabaseTabWidget(QWidget* parent)
             m_dbWidgetStateSync, SLOT(setActive(DatabaseWidget*)));
     connect(autoType(), SIGNAL(globalAutoTypeTriggered(const QString&)), SLOT(performGlobalAutoType(const QString&)));
     connect(autoType(), SIGNAL(autotypeRetypeTimeout()), SLOT(relockPendingDatabase()));
-    connect(autoType(), SIGNAL(autotypeRejected()), SLOT(relockPendingDatabase()));
+    connect(autoType(), SIGNAL(autotypeFinished()), SLOT(relockPendingDatabase()));
     connect(m_databaseOpenDialog.data(), &DatabaseOpenDialog::dialogFinished,
             this, &DatabaseTabWidget::handleDatabaseUnlockDialogFinished);
     // clang-format on
@@ -251,24 +253,53 @@ void DatabaseTabWidget::addDatabaseTab(DatabaseWidget* dbWidget, bool inBackgrou
     connect(dbWidget, SIGNAL(databaseLocked()), SLOT(emitDatabaseLockChanged()));
 }
 
-void DatabaseTabWidget::importCsv()
+DatabaseWidget* DatabaseTabWidget::importFile()
 {
-    auto filter = QString("%1 (*.csv);;%2 (*)").arg(tr("CSV file"), tr("All files"));
-    auto fileName = fileDialog()->getOpenFileName(this, tr("Select CSV file"), FileDialog::getLastDir("csv"), filter);
-    if (fileName.isEmpty()) {
-        return;
+    // Show the import wizard
+    QScopedPointer wizard(new ImportWizard(this));
+    if (!wizard->exec()) {
+        return nullptr;
     }
 
-    FileDialog::saveLastDir("csv", fileName, true);
-
-    auto db = execNewDatabaseWizard();
+    auto db = wizard->database();
     if (!db) {
-        return;
+        // Import wizard was cancelled
+        return nullptr;
     }
 
-    auto* dbWidget = new DatabaseWidget(db, this);
-    addDatabaseTab(dbWidget);
-    dbWidget->switchToCsvImport(fileName);
+    auto importInto = wizard->importInto();
+    if (importInto.first.isNull()) {
+        // Start the new database wizard with the imported database
+        auto newDb = execNewDatabaseWizard();
+        if (newDb) {
+            // Merge the imported db into the new one
+            Merger merger(db.data(), newDb.data());
+            merger.setSkipDatabaseCustomData(true);
+            merger.merge();
+            // Show the new database
+            auto dbWidget = new DatabaseWidget(newDb, this);
+            addDatabaseTab(dbWidget);
+            newDb->markAsModified();
+            return dbWidget;
+        }
+    } else {
+        for (int i = 0, c = count(); i < c; ++i) {
+            // Find the database and group to import into based on import wizard choice
+            auto dbWidget = databaseWidgetFromIndex(i);
+            if (!dbWidget->isLocked() && dbWidget->database()->uuid() == importInto.first) {
+                auto group = dbWidget->database()->rootGroup()->findGroupByUuid(importInto.second);
+                if (group) {
+                    // Extract the root group from the import database
+                    auto importGroup = db->setRootGroup(new Group());
+                    importGroup->setParent(group);
+                    setCurrentIndex(i);
+                    return dbWidget;
+                }
+            }
+        }
+    }
+
+    return nullptr;
 }
 
 void DatabaseTabWidget::mergeDatabase()
@@ -290,44 +321,6 @@ void DatabaseTabWidget::mergeDatabase(const QString& filePath)
     unlockDatabaseInDialog(currentDatabaseWidget(), DatabaseOpenDialog::Intent::Merge, filePath);
 }
 
-void DatabaseTabWidget::importKeePass1Database()
-{
-    auto filter = QString("%1 (*.kdb);;%2 (*)").arg(tr("KeePass 1 database"), tr("All files"));
-    auto fileName =
-        fileDialog()->getOpenFileName(this, tr("Open KeePass 1 database"), FileDialog::getLastDir("kp1"), filter);
-    if (fileName.isEmpty()) {
-        return;
-    }
-
-    FileDialog::saveLastDir("kp1", fileName, true);
-
-    auto db = QSharedPointer<Database>::create();
-    auto* dbWidget = new DatabaseWidget(db, this);
-    addDatabaseTab(dbWidget);
-    dbWidget->switchToImportKeepass1(fileName);
-}
-
-void DatabaseTabWidget::importOpVaultDatabase()
-{
-    auto defaultDir = FileDialog::getLastDir("opvault");
-#ifdef Q_OS_MACOS
-    QString fileName = fileDialog()->getOpenFileName(this, tr("Open OPVault"), defaultDir, "OPVault (*.opvault)");
-#else
-    QString fileName = fileDialog()->getExistingDirectory(this, tr("Open OPVault"), defaultDir);
-#endif
-
-    if (fileName.isEmpty()) {
-        return;
-    }
-
-    FileDialog::saveLastDir("opvault", fileName);
-
-    auto db = QSharedPointer<Database>::create();
-    auto* dbWidget = new DatabaseWidget(db, this);
-    addDatabaseTab(dbWidget);
-    dbWidget->switchToImportOpVault(fileName);
-}
-
 /**
  * Attempt to close the current database and remove its tab afterwards.
  *
@@ -557,6 +550,28 @@ void DatabaseTabWidget::showDatabaseSettings()
     currentDatabaseWidget()->switchToDatabaseSettings();
 }
 
+#ifdef WITH_XC_BROWSER_PASSKEYS
+void DatabaseTabWidget::showPasskeys()
+{
+    currentDatabaseWidget()->switchToPasskeys();
+}
+
+void DatabaseTabWidget::importPasskey()
+{
+    currentDatabaseWidget()->showImportPasskeyDialog();
+}
+
+void DatabaseTabWidget::importPasskeyToEntry()
+{
+    currentDatabaseWidget()->showImportPasskeyDialog(true);
+}
+
+void DatabaseTabWidget::removePasskeyFromEntry()
+{
+    currentDatabaseWidget()->removePasskeyFromEntry();
+}
+#endif
+
 bool DatabaseTabWidget::isModified(int index) const
 {
     if (count() == 0) {
@@ -595,43 +610,18 @@ bool DatabaseTabWidget::hasLockableDatabases() const
  */
 QString DatabaseTabWidget::tabName(int index)
 {
-    if (index == -1 || index > count()) {
-        return "";
+    auto dbWidget = databaseWidgetFromIndex(index);
+    if (!dbWidget) {
+        return {};
     }
 
-    auto* dbWidget = databaseWidgetFromIndex(index);
-
-    auto db = dbWidget->database();
-    Q_ASSERT(db);
-    if (!db) {
-        return "";
-    }
-
-    QString tabName;
-
-    if (!db->filePath().isEmpty()) {
-        QFileInfo fileInfo(db->filePath());
-
-        if (db->metadata()->name().isEmpty()) {
-            tabName = fileInfo.fileName();
-        } else {
-            tabName = db->metadata()->name();
-        }
-
-        setTabToolTip(index, fileInfo.absoluteFilePath());
-    } else {
-        if (db->metadata()->name().isEmpty()) {
-            tabName = tr("New Database");
-        } else {
-            tabName = tr("%1 [New Database]", "Database tab name modifier").arg(db->metadata()->name());
-        }
-    }
+    auto tabName = dbWidget->displayName();
 
     if (dbWidget->isLocked()) {
         tabName = tr("%1 [Locked]", "Database tab name modifier").arg(tabName);
     }
 
-    if (db->isModified()) {
+    if (dbWidget->database()->isModified()) {
         tabName.append("*");
     }
 
@@ -654,6 +644,7 @@ void DatabaseTabWidget::updateTabName(int index)
     }
     index = indexOf(dbWidget);
     setTabText(index, tabName(index));
+    setTabToolTip(index, dbWidget->displayFilePath());
     emit tabNameChanged();
 }
 
@@ -859,6 +850,7 @@ void DatabaseTabWidget::emitDatabaseLockChanged()
         emit databaseLocked(dbWidget);
     } else {
         emit databaseUnlocked(dbWidget);
+        m_databaseOpenInProgress = false;
     }
 }
 
@@ -891,6 +883,11 @@ void DatabaseTabWidget::performGlobalAutoType(const QString& search)
 
 void DatabaseTabWidget::performBrowserUnlock()
 {
+    if (m_databaseOpenInProgress) {
+        return;
+    }
+
+    m_databaseOpenInProgress = true;
     auto dbWidget = currentDatabaseWidget();
     if (dbWidget && dbWidget->isLocked()) {
         unlockAnyDatabaseInDialog(DatabaseOpenDialog::Intent::Browser);
