@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
+ * Copyright (C) 2024 KeePassXC Team <team@keepassxc.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #define KEEPASSX_DATABASETABWIDGET_H
 
 #include "DatabaseOpenDialog.h"
+#include "config-keepassx.h"
 #include "gui/MessageWidget.h"
 
 #include <QTabWidget>
@@ -63,9 +64,7 @@ public slots:
     DatabaseWidget* newDatabase();
     void openDatabase();
     void mergeDatabase();
-    void importCsv();
-    void importKeePass1Database();
-    void importOpVaultDatabase();
+    DatabaseWidget* importFile();
     bool saveDatabase(int index = -1);
     bool saveDatabaseAs(int index = -1);
     bool saveDatabaseBackup(int index = -1);
@@ -84,6 +83,12 @@ public slots:
     void showDatabaseSecurity();
     void showDatabaseReports();
     void showDatabaseSettings();
+#ifdef WITH_XC_BROWSER_PASSKEYS
+    void showPasskeys();
+    void importPasskey();
+    void importPasskeyToEntry();
+    void removePasskeyFromEntry();
+#endif
     void performGlobalAutoType(const QString& search);
     void performBrowserUnlock();
 
@@ -117,6 +122,7 @@ private:
     QPointer<DatabaseWidget> m_dbWidgetPendingLock;
     QPointer<DatabaseOpenDialog> m_databaseOpenDialog;
     QTimer m_lockDelayTimer;
+    bool m_databaseOpenInProgress;
 };
 
 #endif // KEEPASSX_DATABASETABWIDGET_H
