@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2023 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2024 KeePassXC Team <team@keepassxc.org>
  *  Copyright (C) 2010 Felix Geyer <debfx@fobos.de>
  *
  *  This program is free software: you can redistribute it and/or modify
@@ -77,6 +77,28 @@
 #include "mainwindowadaptor.h"
 #endif
 
+// This filter gets installed on all the QAction objects within the MainWindow.
+bool ActionEventFilter::eventFilter(QObject* watched, QEvent* event)
+{
+    auto databaseWidget = getMainWindow()->m_ui->tabWidget->currentDatabaseWidget();
+    if (databaseWidget && event->type() == QEvent::Shortcut) {
+        // We check if we got a Shortcut event that uses the same key sequence as the
+        // OS default copy-to-clipboard shortcut.
+        static const auto stdCopyShortcuts = QKeySequence::keyBindings(QKeySequence::Copy);
+        if (stdCopyShortcuts.contains(static_cast<QShortcutEvent*>(event)->key())) {
+            // If so, we ask the database widget to check if any of its sub-widgets has text
+            // selected, and to copy it to the clipboard if that is the case. We do this
+            // because that is what the user likely expects to happen, yet Qt does not
+            // behave like that on all platforms.
+            if (databaseWidget->copyFocusedTextSelection()) {
+                // In that case, we return true to stop further processing of this event.
+                return true;
+            }
+        }
+    }
+    return QObject::eventFilter(watched, event);
+}
+
 const QString MainWindow::BaseWindowTitle = "KeePassXC";
 
 MainWindow* g_MainWindow = nullptr;
@@ -135,6 +157,11 @@ MainWindow::MainWindow()
     m_entryContextMenu->addSeparator();
     m_entryContextMenu->addAction(m_ui->actionEntryAutoType);
     m_entryContextMenu->addSeparator();
+#ifdef WITH_XC_BROWSER_PASSKEYS
+    m_entryContextMenu->addAction(m_ui->actionEntryImportPasskey);
+    m_entryContextMenu->addAction(m_ui->actionEntryRemovePasskey);
+    m_entryContextMenu->addSeparator();
+#endif
     m_entryContextMenu->addAction(m_ui->actionEntryEdit);
     m_entryContextMenu->addAction(m_ui->actionEntryClone);
     m_entryContextMenu->addAction(m_ui->actionEntryDelete);
@@ -384,7 +411,7 @@ MainWindow::MainWindow()
     m_ui->actionLockAllDatabases->setIcon(icons()->icon("database-lock-all"));
     m_ui->actionQuit->setIcon(icons()->icon("application-exit"));
     m_ui->actionDatabaseMerge->setIcon(icons()->icon("database-merge"));
-    m_ui->menuImport->setIcon(icons()->icon("document-import"));
+    m_ui->actionImport->setIcon(icons()->icon("document-import"));
     m_ui->menuExport->setIcon(icons()->icon("document-export"));
 
     m_ui->actionEntryNew->setIcon(icons()->icon("entry-new"));
@@ -438,6 +465,16 @@ MainWindow::MainWindow()
     m_ui->actionKeyboardShortcuts->setIcon(icons()->icon("keyboard-shortcuts"));
     m_ui->actionCheckForUpdates->setIcon(icons()->icon("system-software-update"));
 
+#ifdef WITH_XC_BROWSER_PASSKEYS
+    m_ui->actionPasskeys->setIcon(icons()->icon("passkey"));
+    m_ui->actionImportPasskey->setIcon(icons()->icon("document-import"));
+    m_ui->actionEntryImportPasskey->setIcon(icons()->icon("document-import"));
+    m_ui->actionEntryRemovePasskey->setIcon(icons()->icon("document-close"));
+#endif
+
+    // Handle copy to clipboard shortcut interference
+    m_ui->actionEntryCopyPassword->installEventFilter(&m_actionEventFilter);
+
     m_actionMultiplexer.connect(
         SIGNAL(currentModeChanged(DatabaseWidget::Mode)), this, SLOT(setMenuActionState(DatabaseWidget::Mode)));
     m_actionMultiplexer.connect(SIGNAL(groupChanged()), this, SLOT(setMenuActionState()));
@@ -483,9 +520,13 @@ MainWindow::MainWindow()
     connect(m_ui->actionDatabaseSecurity, SIGNAL(triggered()), m_ui->tabWidget, SLOT(showDatabaseSecurity()));
     connect(m_ui->actionReports, SIGNAL(triggered()), m_ui->tabWidget, SLOT(showDatabaseReports()));
     connect(m_ui->actionDatabaseSettings, SIGNAL(triggered()), m_ui->tabWidget, SLOT(showDatabaseSettings()));
-    connect(m_ui->actionImportCsv, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importCsv()));
-    connect(m_ui->actionImportKeePass1, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importKeePass1Database()));
-    connect(m_ui->actionImportOpVault, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importOpVaultDatabase()));
+#ifdef WITH_XC_BROWSER_PASSKEYS
+    connect(m_ui->actionPasskeys, SIGNAL(triggered()), m_ui->tabWidget, SLOT(showPasskeys()));
+    connect(m_ui->actionImportPasskey, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importPasskey()));
+    connect(m_ui->actionEntryImportPasskey, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importPasskeyToEntry()));
+    connect(m_ui->actionEntryRemovePasskey, SIGNAL(triggered()), m_ui->tabWidget, SLOT(removePasskeyFromEntry()));
+#endif
+    connect(m_ui->actionImport, SIGNAL(triggered()), m_ui->tabWidget, SLOT(importFile()));
     connect(m_ui->actionExportCsv, SIGNAL(triggered()), m_ui->tabWidget, SLOT(exportToCsv()));
     connect(m_ui->actionExportHtml, SIGNAL(triggered()), m_ui->tabWidget, SLOT(exportToHtml()));
     connect(m_ui->actionExportXML, SIGNAL(triggered()), m_ui->tabWidget, SLOT(exportToXML()));
@@ -551,9 +592,7 @@ MainWindow::MainWindow()
     connect(m_ui->welcomeWidget, SIGNAL(newDatabase()), SLOT(switchToNewDatabase()));
     connect(m_ui->welcomeWidget, SIGNAL(openDatabase()), SLOT(switchToOpenDatabase()));
     connect(m_ui->welcomeWidget, SIGNAL(openDatabaseFile(QString)), SLOT(switchToDatabaseFile(QString)));
-    connect(m_ui->welcomeWidget, SIGNAL(importKeePass1Database()), SLOT(switchToKeePass1Database()));
-    connect(m_ui->welcomeWidget, SIGNAL(importOpVaultDatabase()), SLOT(switchToOpVaultDatabase()));
-    connect(m_ui->welcomeWidget, SIGNAL(importCsv()), SLOT(switchToCsvImport()));
+    connect(m_ui->welcomeWidget, SIGNAL(importFile()), m_ui->tabWidget, SLOT(importFile()));
 
     connect(m_ui->actionAbout, SIGNAL(triggered()), SLOT(showAboutDialog()));
     connect(m_ui->actionDonate, SIGNAL(triggered()), SLOT(openDonateUrl()));
@@ -572,6 +611,7 @@ MainWindow::MainWindow()
     m_ui->menubar->installEventFilter(eventFilter);
     m_ui->toolBar->installEventFilter(eventFilter);
     m_ui->tabWidget->tabBar()->installEventFilter(eventFilter);
+    installEventFilter(eventFilter);
 #endif
 
 #ifdef Q_OS_MACOS
@@ -882,7 +922,7 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
     m_ui->actionDatabaseNew->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
     m_ui->actionDatabaseOpen->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
     m_ui->menuRecentDatabases->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
-    m_ui->menuImport->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
+    m_ui->actionImport->setEnabled(inDatabaseTabWidgetOrWelcomeWidget);
     m_ui->actionLockDatabase->setEnabled(m_ui->tabWidget->hasLockableDatabases());
     m_ui->actionLockDatabaseToolbar->setEnabled(m_ui->tabWidget->hasLockableDatabases());
     m_ui->actionLockAllDatabases->setEnabled(m_ui->tabWidget->hasLockableDatabases());
@@ -977,6 +1017,13 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             m_ui->actionExportHtml->setEnabled(true);
             m_ui->actionExportXML->setEnabled(true);
             m_ui->actionDatabaseMerge->setEnabled(m_ui->tabWidget->currentIndex() != -1);
+#ifdef WITH_XC_BROWSER_PASSKEYS
+            bool singleEntryHasPasskey = singleEntrySelected && dbWidget->currentEntryHasPasskey();
+            m_ui->actionPasskeys->setEnabled(true);
+            m_ui->actionImportPasskey->setEnabled(true);
+            m_ui->actionEntryImportPasskey->setEnabled(singleEntrySelected);
+            m_ui->actionEntryRemovePasskey->setEnabled(singleEntryHasPasskey);
+#endif
 #ifdef WITH_XC_SSHAGENT
             bool singleEntryHasSshKey =
                 singleEntrySelected && sshAgent()->isEnabled() && dbWidget->currentEntryHasSshKey();
@@ -991,7 +1038,6 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             break;
         }
         case DatabaseWidget::Mode::EditMode:
-        case DatabaseWidget::Mode::ImportMode:
         case DatabaseWidget::Mode::LockedMode: {
             // Enable select actions when editing an entry
             bool editEntryActive = dbWidget->isEntryEditActive();
@@ -1044,6 +1090,18 @@ void MainWindow::setMenuActionState(DatabaseWidget::Mode mode)
             m_ui->actionEntryRemoveFromAgent->setVisible(false);
             m_ui->actionGroupEmptyRecycleBin->setVisible(false);
 
+#ifdef WITH_XC_BROWSER_PASSKEYS
+            m_ui->actionPasskeys->setEnabled(false);
+            m_ui->actionImportPasskey->setEnabled(false);
+            m_ui->actionEntryImportPasskey->setEnabled(false);
+            m_ui->actionEntryRemovePasskey->setEnabled(false);
+#else
+            m_ui->actionPasskeys->setVisible(false);
+            m_ui->actionImportPasskey->setVisible(false);
+            m_ui->actionEntryImportPasskey->setVisible(false);
+            m_ui->actionEntryRemovePasskey->setVisible(false);
+#endif
+
             m_searchWidgetAction->setEnabled(false);
             break;
         }
@@ -1213,7 +1271,11 @@ void MainWindow::showUpdateCheckDialog()
 
 void MainWindow::customOpenUrl(QString url)
 {
+#ifdef KEEPASSXC_DIST_APPIMAGE
+    QProcess::execute("xdg-open", {url});
+#else
     QDesktopServices::openUrl(QUrl(url));
+#endif
 }
 
 void MainWindow::openDonateUrl()
@@ -1295,24 +1357,6 @@ void MainWindow::switchToDatabaseFile(const QString& file)
     switchToDatabases();
 }
 
-void MainWindow::switchToKeePass1Database()
-{
-    m_ui->tabWidget->importKeePass1Database();
-    switchToDatabases();
-}
-
-void MainWindow::switchToOpVaultDatabase()
-{
-    m_ui->tabWidget->importOpVaultDatabase();
-    switchToDatabases();
-}
-
-void MainWindow::switchToCsvImport()
-{
-    m_ui->tabWidget->importCsv();
-    switchToDatabases();
-}
-
 void MainWindow::databaseStatusChanged(DatabaseWidget* dbWidget)
 {
     Q_UNUSED(dbWidget);
@@ -1668,6 +1712,7 @@ void MainWindow::applySettingsChanges()
         m_inactivityTimer->deactivate();
     }
 
+    m_ui->menubar->setHidden(config()->get(Config::GUI_HideMenubar).toBool());
     m_ui->toolBar->setHidden(config()->get(Config::GUI_HideToolbar).toBool());
     auto movable = config()->get(Config::GUI_MovableToolbar).toBool();
     m_ui->toolBar->setMovable(movable);
@@ -1851,7 +1896,9 @@ void MainWindow::closeModalWindow()
 
 void MainWindow::lockDatabasesAfterInactivity()
 {
-    m_ui->tabWidget->lockDatabases();
+    if (!m_ui->tabWidget->lockDatabases()) {
+        m_inactivityTimer->activate();
+    }
 }
 
 bool MainWindow::isTrayIconEnabled() const
@@ -2030,6 +2077,16 @@ void MainWindow::initViewMenu()
         }
     });
 
+#ifdef Q_OS_MACOS
+    m_ui->actionShowMenubar->setVisible(false);
+#else
+    m_ui->actionShowMenubar->setChecked(!config()->get(Config::GUI_HideMenubar).toBool());
+    connect(m_ui->actionShowMenubar, &QAction::toggled, this, [this](bool checked) {
+        config()->set(Config::GUI_HideMenubar, !checked);
+        applySettingsChanges();
+    });
+#endif
+
     m_ui->actionShowToolbar->setChecked(!config()->get(Config::GUI_HideToolbar).toBool());
     connect(m_ui->actionShowToolbar, &QAction::toggled, this, [this](bool checked) {
         config()->set(Config::GUI_HideToolbar, !checked);
@@ -2073,6 +2130,7 @@ MainWindowEventFilter::MainWindowEventFilter(QObject* parent)
 
 /**
  * MainWindow event filter to initiate empty-area drag on the toolbar, menubar, and tabbar.
+ * Also shows menubar with Alt when menubar itself is hidden.
  */
 bool MainWindowEventFilter::eventFilter(QObject* watched, QEvent* event)
 {
@@ -2081,10 +2139,11 @@ bool MainWindowEventFilter::eventFilter(QObject* watched, QEvent* event)
         return QObject::eventFilter(watched, event);
     }
 
-    if (event->type() == QEvent::MouseButtonPress) {
+    auto eventType = event->type();
+    if (eventType == QEvent::MouseButtonPress) {
+        auto mouseEvent = dynamic_cast<QMouseEvent*>(event);
         if (watched == mainWindow->m_ui->menubar) {
-            auto* m = static_cast<QMouseEvent*>(event);
-            if (!mainWindow->m_ui->menubar->actionAt(m->pos())) {
+            if (!mainWindow->m_ui->menubar->actionAt(mouseEvent->pos())) {
                 mainWindow->windowHandle()->startSystemMove();
                 return false;
             }
@@ -2094,12 +2153,24 @@ bool MainWindowEventFilter::eventFilter(QObject* watched, QEvent* event)
                 return false;
             }
         } else if (watched == mainWindow->m_ui->tabWidget->tabBar()) {
-            auto* m = static_cast<QMouseEvent*>(event);
-            if (mainWindow->m_ui->tabWidget->tabBar()->tabAt(m->pos()) == -1) {
+            if (mainWindow->m_ui->tabWidget->tabBar()->tabAt(mouseEvent->pos()) == -1) {
                 mainWindow->windowHandle()->startSystemMove();
                 return true;
             }
         }
+    } else if (eventType == QEvent::KeyRelease) {
+        if (watched == mainWindow) {
+            auto keyEvent = dynamic_cast<QKeyEvent*>(event);
+            if (keyEvent->key() == Qt::Key_Alt && !keyEvent->modifiers()
+                && config()->get(Config::GUI_HideMenubar).toBool()) {
+                auto menubar = mainWindow->m_ui->menubar;
+                menubar->setVisible(!menubar->isVisible());
+                if (menubar->isVisible()) {
+                    menubar->setActiveAction(mainWindow->m_ui->menuFile->menuAction());
+                }
+                return false;
+            }
+        }
     }
 
     return QObject::eventFilter(watched, event);
