@@ -33,6 +33,7 @@ class PasswordWidget : public QWidget
 {
     Q_OBJECT
 
+    Q_PROPERTY(QString text READ text WRITE setText NOTIFY textChanged USER true)
 public:
     explicit PasswordWidget(QWidget* parent = nullptr);
     ~PasswordWidget() override;
@@ -60,7 +61,6 @@ protected:
     bool event(QEvent* event) override;
 
 private slots:
-    void autocompletePassword(const QString& password);
     void popupPasswordGenerator();
     void updateRepeatStatus();
     void updatePasswordStrength(const QString& password);
