@@ -23,11 +23,13 @@
 class SquareSvgWidget : public QSvgWidget
 {
 public:
-    SquareSvgWidget() = default;
+    explicit SquareSvgWidget(QWidget* parent);
     ~SquareSvgWidget() override = default;
 
     bool hasHeightForWidth() const override;
     int heightForWidth(int width) const override;
+
+    void resizeEvent(QResizeEvent* event) override;
 };
 
 #endif // KEEPASSX_SquareSvgWidget_H
