@@ -27,6 +27,9 @@ PasswordEditWidget::PasswordEditWidget(QWidget* parent)
     , m_compUi(new Ui::PasswordEditWidget())
 {
     initComponent();
+
+    // Explicitly clear password on cancel
+    connect(this, &PasswordEditWidget::editCanceled, this, [this] { setPassword({}); });
 }
 
 PasswordEditWidget::~PasswordEditWidget()
@@ -61,7 +64,15 @@ bool PasswordEditWidget::isPasswordVisible() const
 
 bool PasswordEditWidget::isEmpty() const
 {
-    return (visiblePage() == Page::Edit) && m_compUi->enterPasswordEdit->text().isEmpty();
+    return visiblePage() != Page::Edit || m_compUi->enterPasswordEdit->text().isEmpty();
+}
+
+PasswordHealth::Quality PasswordEditWidget::getPasswordQuality() const
+{
+    QString pwd = m_compUi->enterPasswordEdit->text();
+    PasswordHealth passwordHealth(pwd);
+
+    return passwordHealth.quality();
 }
 
 QWidget* PasswordEditWidget::componentEditWidget()
@@ -77,8 +88,7 @@ void PasswordEditWidget::initComponentEditWidget(QWidget* widget)
 {
     Q_UNUSED(widget);
     Q_ASSERT(m_compEditWidget);
-    m_compUi->enterPasswordEdit->setFocus();
-
+    setFocusProxy(m_compUi->enterPasswordEdit);
     m_compUi->enterPasswordEdit->setQualityVisible(true);
     m_compUi->repeatPasswordEdit->setQualityVisible(false);
 }
@@ -97,16 +107,6 @@ void PasswordEditWidget::initComponent()
            "<p>Good passwords are long and unique. KeePassXC can generate one for you.</p>"));
 }
 
-void PasswordEditWidget::hideEvent(QHideEvent* event)
-{
-    if (!isVisible() && m_compUi->enterPasswordEdit) {
-        m_compUi->enterPasswordEdit->setText("");
-        m_compUi->repeatPasswordEdit->setText("");
-    }
-
-    QWidget::hideEvent(event);
-}
-
 bool PasswordEditWidget::validate(QString& errorMessage) const
 {
     if (m_compUi->enterPasswordEdit->text() != m_compUi->repeatPasswordEdit->text()) {
