@@ -33,7 +33,6 @@
 #include "fdosecrets/DatabaseSettingsPageFdoSecrets.h"
 #endif
 
-#include "core/Config.h"
 #include "core/Database.h"
 #include "core/Global.h"
 #include "gui/Icons.h"
@@ -92,34 +91,29 @@ DatabaseSettingsDialog::DatabaseSettingsDialog(QWidget* parent)
     scrollArea->setSizeAdjustPolicy(QScrollArea::AdjustToContents);
     scrollArea->setWidgetResizable(true);
     scrollArea->setWidget(m_databaseKeyWidget);
+
     m_securityTabWidget->addTab(scrollArea, tr("Database Credentials"));
-
     m_securityTabWidget->addTab(m_encryptionWidget, tr("Encryption Settings"));
-
-#if defined(WITH_XC_KEESHARE)
-    addSettingsPage(new DatabaseSettingsPageKeeShare());
-#endif
-
-#if defined(WITH_XC_FDOSECRETS)
-    addSettingsPage(new DatabaseSettingsPageFdoSecrets());
-#endif
-
-    m_ui->stackedWidget->setCurrentIndex(0);
     m_securityTabWidget->setCurrentIndex(0);
 
-    connect(m_securityTabWidget, SIGNAL(currentChanged(int)), SLOT(pageChanged()));
-    connect(m_ui->categoryList, SIGNAL(categoryChanged(int)), m_ui->stackedWidget, SLOT(setCurrentIndex(int)));
-    connect(m_ui->advancedSettingsToggle, SIGNAL(toggled(bool)), SLOT(toggleAdvancedMode(bool)));
-
 #ifdef WITH_XC_BROWSER
     m_ui->categoryList->addCategory(tr("Browser Integration"), icons()->icon("internet-web-browser"));
     m_ui->stackedWidget->addWidget(m_browserWidget);
 #endif
 
+#ifdef WITH_XC_KEESHARE
+    addSettingsPage(new DatabaseSettingsPageKeeShare());
+#endif
+
+#ifdef WITH_XC_FDOSECRETS
+    addSettingsPage(new DatabaseSettingsPageFdoSecrets());
+#endif
+
     m_ui->categoryList->addCategory(tr("Maintenance"), icons()->icon("hammer-wrench"));
     m_ui->stackedWidget->addWidget(m_maintenanceWidget);
 
-    pageChanged();
+    m_ui->stackedWidget->setCurrentIndex(0);
+    connect(m_ui->categoryList, SIGNAL(categoryChanged(int)), m_ui->stackedWidget, SLOT(setCurrentIndex(int)));
 }
 
 DatabaseSettingsDialog::~DatabaseSettingsDialog()
@@ -139,7 +133,6 @@ void DatabaseSettingsDialog::load(const QSharedPointer<Database>& db)
     for (const ExtraPage& page : asConst(m_extraPages)) {
         page.loadSettings(db);
     }
-    m_ui->advancedSettingsToggle->setChecked(config()->get(Config::GUI_AdvancedSettings).toBool());
     m_db = db;
 }
 
@@ -166,17 +159,24 @@ void DatabaseSettingsDialog::showDatabaseKeySettings()
 void DatabaseSettingsDialog::save()
 {
     if (!m_generalWidget->save()) {
+        m_ui->categoryList->setCurrentCategory(0);
         return;
     }
 
     if (!m_databaseKeyWidget->save()) {
+        m_ui->categoryList->setCurrentCategory(1);
+        m_securityTabWidget->setCurrentIndex(0);
         return;
     }
 
     if (!m_encryptionWidget->save()) {
+        m_ui->categoryList->setCurrentCategory(1);
+        m_securityTabWidget->setCurrentIndex(1);
         return;
     }
 
+    // Browser settings don't have anything to save
+
     for (const ExtraPage& extraPage : asConst(m_extraPages)) {
         extraPage.saveSettings();
     }
@@ -186,37 +186,12 @@ void DatabaseSettingsDialog::save()
 
 void DatabaseSettingsDialog::reject()
 {
+    m_generalWidget->discard();
+    m_databaseKeyWidget->discard();
+    m_encryptionWidget->discard();
+#ifdef WITH_XC_BROWSER
+    m_browserWidget->discard();
+#endif
+
     emit editFinished(false);
 }
-
-void DatabaseSettingsDialog::pageChanged()
-{
-    int pageIndex = m_ui->stackedWidget->currentIndex();
-
-    bool enabled = (pageIndex == Page::General && m_generalWidget->hasAdvancedMode());
-
-    if (Page::Security == pageIndex) {
-        int tabIndex = m_securityTabWidget->currentIndex();
-        enabled = (tabIndex == 0 && m_databaseKeyWidget->hasAdvancedMode());
-        enabled |= (tabIndex == 1 && m_encryptionWidget->hasAdvancedMode());
-    }
-
-    m_ui->advancedSettingsToggle->setEnabled(enabled);
-}
-
-void DatabaseSettingsDialog::toggleAdvancedMode(bool advanced)
-{
-    if (m_generalWidget->hasAdvancedMode()) {
-        m_generalWidget->setAdvancedMode(advanced);
-    }
-
-    if (m_databaseKeyWidget->hasAdvancedMode()) {
-        m_databaseKeyWidget->setAdvancedMode(advanced);
-    }
-
-    if (m_encryptionWidget->hasAdvancedMode()) {
-        m_encryptionWidget->setAdvancedMode(advanced);
-    }
-
-    config()->set(Config::GUI_AdvancedSettings, advanced);
-}
