@@ -17,22 +17,30 @@
 
 #include "DatabaseSettingsWidgetDatabaseKey.h"
 
+#include "core/Config.h"
 #include "core/Database.h"
+#include "core/PasswordHealth.h"
 #include "gui/MessageBox.h"
 #include "gui/databasekey/KeyFileEditWidget.h"
 #include "gui/databasekey/PasswordEditWidget.h"
 #include "gui/databasekey/YubiKeyEditWidget.h"
+#include "keys/ChallengeResponseKey.h"
 #include "keys/FileKey.h"
 #include "keys/PasswordKey.h"
-#include "keys/YkChallengeResponseKey.h"
 
+#ifdef Q_OS_MACOS
+#include "touchid/TouchID.h"
+#endif
+#ifdef Q_CC_MSVC
+#include "winhello/WindowsHello.h"
+#endif
+
+#include <QLayout>
 #include <QPushButton>
-#include <QSpacerItem>
-#include <QVBoxLayout>
 
 DatabaseSettingsWidgetDatabaseKey::DatabaseSettingsWidgetDatabaseKey(QWidget* parent)
     : DatabaseSettingsWidget(parent)
-    , m_additionalKeyOptionsToggle(new QPushButton(tr("Add additional protection..."), this))
+    , m_additionalKeyOptionsToggle(new QPushButton(tr("Add additional protection…"), this))
     , m_additionalKeyOptions(new QWidget(this))
     , m_passwordEditWidget(new PasswordEditWidget(this))
     , m_keyFileEditWidget(new KeyFileEditWidget(this))
@@ -42,6 +50,7 @@ DatabaseSettingsWidgetDatabaseKey::DatabaseSettingsWidgetDatabaseKey(QWidget* pa
 {
     auto* vbox = new QVBoxLayout(this);
     vbox->setSizeConstraint(QLayout::SetMinimumSize);
+    vbox->setSpacing(20);
 
     // primary password option
     vbox->addWidget(m_passwordEditWidget);
@@ -53,6 +62,7 @@ DatabaseSettingsWidgetDatabaseKey::DatabaseSettingsWidgetDatabaseKey(QWidget* pa
     vbox->setSizeConstraint(QLayout::SetMinimumSize);
     m_additionalKeyOptions->setLayout(new QVBoxLayout());
     m_additionalKeyOptions->layout()->setMargin(0);
+    m_additionalKeyOptions->layout()->setSpacing(20);
     m_additionalKeyOptions->layout()->addWidget(m_keyFileEditWidget);
 #ifdef WITH_XC_YUBIKEY
     m_additionalKeyOptions->layout()->addWidget(m_yubiKeyEditWidget);
@@ -77,28 +87,30 @@ void DatabaseSettingsWidgetDatabaseKey::load(QSharedPointer<Database> db)
         // database has no key, we are about to add a new one
         m_passwordEditWidget->changeVisiblePage(KeyComponentWidget::Page::Edit);
         m_passwordEditWidget->setPasswordVisible(true);
-    }
-
-    bool hasAdditionalKeys = false;
-    for (const auto& key : m_db->key()->keys()) {
-        if (key->uuid() == PasswordKey::UUID) {
-            m_passwordEditWidget->setComponentAdded(true);
-        } else if (key->uuid() == FileKey::UUID) {
-            m_keyFileEditWidget->setComponentAdded(true);
-            hasAdditionalKeys = true;
+        // Focus won't work until the UI settles
+        QTimer::singleShot(0, m_passwordEditWidget, SLOT(setFocus()));
+    } else {
+        bool hasAdditionalKeys = false;
+        for (const auto& key : m_db->key()->keys()) {
+            if (key->uuid() == PasswordKey::UUID) {
+                m_passwordEditWidget->setComponentAdded(true);
+            } else if (key->uuid() == FileKey::UUID) {
+                m_keyFileEditWidget->setComponentAdded(true);
+                hasAdditionalKeys = true;
+            }
         }
-    }
 
 #ifdef WITH_XC_YUBIKEY
-    for (const auto& key : m_db->key()->challengeResponseKeys()) {
-        if (key->uuid() == YkChallengeResponseKey::UUID) {
-            m_yubiKeyEditWidget->setComponentAdded(true);
-            hasAdditionalKeys = true;
+        for (const auto& key : m_db->key()->challengeResponseKeys()) {
+            if (key->uuid() == ChallengeResponseKey::UUID) {
+                m_yubiKeyEditWidget->setComponentAdded(true);
+                hasAdditionalKeys = true;
+            }
         }
-    }
 #endif
 
-    setAdditionalKeyOptionsVisible(hasAdditionalKeys);
+        setAdditionalKeyOptionsVisible(hasAdditionalKeys);
+    }
 
     connect(m_passwordEditWidget->findChild<QPushButton*>("removeButton"), SIGNAL(clicked()), SLOT(markDirty()));
     connect(m_keyFileEditWidget->findChild<QPushButton*>("removeButton"), SIGNAL(clicked()), SLOT(markDirty()));
@@ -150,11 +162,12 @@ bool DatabaseSettingsWidgetDatabaseKey::save()
     }
 
     for (const auto& key : m_db->key()->challengeResponseKeys()) {
-        if (key->uuid() == YkChallengeResponseKey::UUID) {
+        if (key->uuid() == ChallengeResponseKey::UUID) {
             oldChallengeResponse = key;
         }
     }
 
+    // Show warning if database password has not been set
     if (m_passwordEditWidget->visiblePage() == KeyComponentWidget::Page::AddNew || m_passwordEditWidget->isEmpty()) {
         QScopedPointer<QMessageBox> msgBox(new QMessageBox(this));
         msgBox->setIcon(QMessageBox::Warning);
@@ -165,6 +178,7 @@ bool DatabaseSettingsWidgetDatabaseKey::save()
         auto btn = msgBox->addButton(tr("Continue without password"), QMessageBox::ButtonRole::AcceptRole);
         msgBox->addButton(QMessageBox::Cancel);
         msgBox->setDefaultButton(QMessageBox::Cancel);
+        msgBox->layout()->setSizeConstraint(QLayout::SetMinimumSize);
         msgBox->exec();
         if (msgBox->clickedButton() != btn) {
             return false;
@@ -173,6 +187,34 @@ bool DatabaseSettingsWidgetDatabaseKey::save()
         return false;
     }
 
+    if (!m_passwordEditWidget->isEmpty()) {
+        // Prevent setting password with a quality less than the minimum required
+        auto minQuality = qBound(0, config()->get(Config::Security_DatabasePasswordMinimumQuality).toInt(), 4);
+        if (m_passwordEditWidget->getPasswordQuality() < static_cast<PasswordHealth::Quality>(minQuality)) {
+            MessageBox::critical(this,
+                                 tr("Weak password"),
+                                 tr("The provided password does not meet the minimum quality requirement."),
+                                 MessageBox::Ok,
+                                 MessageBox::Ok);
+            return false;
+        }
+
+        // Show warning if database password is weak or poor
+        if (m_passwordEditWidget->getPasswordQuality() < PasswordHealth::Quality::Good) {
+            auto dialogResult =
+                MessageBox::warning(this,
+                                    tr("Weak password"),
+                                    tr("This is a weak password! For better protection of your secrets, "
+                                       "you should choose a stronger password."),
+                                    MessageBox::ContinueWithWeakPass | MessageBox::Cancel,
+                                    MessageBox::Cancel);
+
+            if (dialogResult == MessageBox::Cancel) {
+                return false;
+            }
+        }
+    }
+
     if (!addToCompositeKey(m_keyFileEditWidget, newKey, oldFileKey)) {
         return false;
     }
@@ -194,16 +236,27 @@ bool DatabaseSettingsWidgetDatabaseKey::save()
 
     m_db->setKey(newKey, true, false, false);
 
+#if defined(Q_OS_MACOS)
+    TouchID::getInstance().reset(m_db->filePath());
+#elif defined(Q_CC_MSVC)
+    getWindowsHello()->reset(m_db->filePath());
+#endif
+
     emit editFinished(true);
     if (m_isDirty) {
         m_db->markAsModified();
     }
 
+    // Reset fields
+    initialize();
+
     return true;
 }
 
 void DatabaseSettingsWidgetDatabaseKey::discard()
 {
+    // Reset fields
+    initialize();
     emit editFinished(false);
 }
 
