@@ -1,3 +1,20 @@
+/*
+ *  Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 2 or (at your option)
+ *  version 3 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef ENTRYATTACHMENTSWIDGET_H
 #define ENTRYATTACHMENTSWIDGET_H
 
@@ -22,17 +39,13 @@ public:
     explicit EntryAttachmentsWidget(QWidget* parent = nullptr);
     ~EntryAttachmentsWidget();
 
-    const EntryAttachments* entryAttachments() const;
+    const EntryAttachments* attachments() const;
     bool isReadOnly() const;
     bool isButtonsVisible() const;
 
-    QByteArray getAttachment(const QString& name);
-    void setAttachment(const QString& name, const QByteArray& value);
-    void removeAttachment(const QString& name);
-
 public slots:
-    void setEntryAttachments(const EntryAttachments* attachments);
-    void clearAttachments();
+    void linkAttachments(EntryAttachments* attachments);
+    void unlinkAttachments();
     void setReadOnly(bool readOnly);
     void setButtonsVisible(bool isButtonsVisible);
 
@@ -45,23 +58,25 @@ signals:
 private slots:
     void insertAttachments();
     void removeSelectedAttachments();
+    void renameSelectedAttachments();
     void saveSelectedAttachments();
     void openAttachment(const QModelIndex& index);
     void openSelectedAttachments();
     void updateButtonsVisible();
     void updateButtonsEnabled();
+    void attachmentModifiedExternally(const QString& key, const QString& filePath);
 
 private:
     bool insertAttachments(const QStringList& fileNames, QString& errorMessage);
-    bool openAttachment(const QModelIndex& index, QString& errorMessage);
 
-    QStringList confirmLargeAttachments(const QStringList& filenames);
+    QStringList confirmAttachmentSelection(const QStringList& filenames);
 
     bool eventFilter(QObject* watched, QEvent* event) override;
 
     QScopedPointer<Ui::EntryAttachmentsWidget> m_ui;
     QPointer<EntryAttachments> m_entryAttachments;
     QPointer<EntryAttachmentsModel> m_attachmentsModel;
+    QStringList m_pendingChanges;
     bool m_readOnly;
     bool m_buttonsVisible;
 };
