@@ -20,6 +20,7 @@
 
 #include <QAbstractTableModel>
 #include <QPixmap>
+#include <QSet>
 
 #include "core/Config.h"
 
@@ -46,7 +47,8 @@ public:
         Paperclip = 10,
         Attachments = 11,
         Totp = 12,
-        Size = 13
+        Size = 13,
+        PasswordStrength = 14
     };
 
     explicit EntryModel(QObject* parent = nullptr);
@@ -86,7 +88,7 @@ private:
     Group* m_group;
     QList<Entry*> m_entries;
     QList<Entry*> m_orgEntries;
-    QList<const Group*> m_allGroups;
+    QSet<const Group*> m_allGroups;
 
     const QString HiddenContentDisplay;
     const Qt::DateFormat DateFormat;
