@@ -22,12 +22,9 @@
 #include "crypto/CryptoHash.h"
 #include "crypto/Random.h"
 
+#include <QDataStream>
 #include <QFile>
-
-#include <algorithm>
-#include <cstring>
-#include <gcrypt.h>
-#include <sodium.h>
+#include <QXmlStreamReader>
 
 QUuid FileKey::UUID("a584cbc4-c9b4-437e-81bb-362ca9709273");
 
@@ -35,18 +32,10 @@ constexpr int FileKey::SHA256_SIZE;
 
 FileKey::FileKey()
     : Key(UUID)
-    , m_key(static_cast<char*>(gcry_malloc_secure(SHA256_SIZE)))
+    , m_key(SHA256_SIZE)
 {
 }
 
-FileKey::~FileKey()
-{
-    if (m_key) {
-        gcry_free(m_key);
-        m_key = nullptr;
-    }
-}
-
 /**
  * Read key file from device while trying to detect its file format.
  *
@@ -159,6 +148,9 @@ bool FileKey::load(const QString& fileName, QString* errorMsg)
         if (errorMsg) {
             *errorMsg = file.errorString();
         }
+    } else {
+        // Store the file path for serialization
+        m_file = fileName;
     }
 
     return result;
@@ -169,10 +161,36 @@ bool FileKey::load(const QString& fileName, QString* errorMsg)
  */
 QByteArray FileKey::rawKey() const
 {
-    if (!m_key) {
-        return {};
+    return QByteArray(m_key.data(), m_key.size());
+}
+
+void FileKey::setRawKey(const QByteArray& data)
+{
+    Q_ASSERT(data.size() == SHA256_SIZE);
+    m_key.assign(data.begin(), data.end());
+}
+
+QByteArray FileKey::serialize() const
+{
+    QByteArray data;
+    QDataStream stream(&data, QIODevice::WriteOnly);
+    stream << uuid().toRfc4122() << rawKey() << static_cast<qint32>(m_type) << m_file;
+    return data;
+}
+
+void FileKey::deserialize(const QByteArray& data)
+{
+    QDataStream stream(data);
+    QByteArray uuidData;
+    stream >> uuidData;
+    if (uuid().toRfc4122() == uuidData) {
+        QByteArray key;
+        qint32 type;
+        stream >> key >> type >> m_file;
+
+        setRawKey(key);
+        m_type = static_cast<Type>(type);
     }
-    return QByteArray::fromRawData(m_key, SHA256_SIZE);
 }
 
 /**
@@ -225,7 +243,7 @@ void FileKey::createXMLv2(QIODevice* device, int size)
         }
         w.writeCharacters(QChar(key[i]));
     }
-    sodium_memzero(key.data(), static_cast<std::size_t>(key.capacity()));
+    Botan::secure_scrub_memory(key.data(), static_cast<std::size_t>(key.capacity()));
     w.writeCharacters("\n        ");
 
     w.writeEndElement();
@@ -315,12 +333,12 @@ bool FileKey::loadXml(QIODevice* device, QString* errorMsg)
             while (!xmlReader.error() && xmlReader.readNextStartElement()) {
                 if (xmlReader.name() == "Data") {
                     keyFileData.hash = QByteArray::fromHex(xmlReader.attributes().value("Hash").toLatin1());
-                    QByteArray rawData = xmlReader.readElementText().simplified().replace(" ", "").toLatin1();
+                    keyFileData.data = xmlReader.readElementText().simplified().replace(" ", "").toLatin1();
 
-                    if (keyFileData.version.startsWith("1.0") && Tools::isBase64(rawData)) {
-                        keyFileData.data = QByteArray::fromBase64(rawData);
-                    } else if (keyFileData.version == "2.0" && Tools::isHex(rawData)) {
-                        keyFileData.data = QByteArray::fromHex(rawData);
+                    if (keyFileData.version.startsWith("1.0") && Tools::isBase64(keyFileData.data)) {
+                        keyFileData.data = QByteArray::fromBase64(keyFileData.data);
+                    } else if (keyFileData.version == "2.0" && Tools::isHex(keyFileData.data)) {
+                        keyFileData.data = QByteArray::fromHex(keyFileData.data);
 
                         CryptoHash hash(CryptoHash::Sha256);
                         hash.addData(keyFileData.data);
@@ -337,8 +355,6 @@ bool FileKey::loadXml(QIODevice* device, QString* errorMsg)
                         }
                         return false;
                     }
-
-                    sodium_memzero(rawData.data(), static_cast<std::size_t>(rawData.capacity()));
                 }
             }
         }
@@ -346,11 +362,11 @@ bool FileKey::loadXml(QIODevice* device, QString* errorMsg)
 
     bool ok = false;
     if (!xmlReader.error() && !keyFileData.data.isEmpty()) {
-        std::memcpy(m_key, keyFileData.data.data(), std::min(SHA256_SIZE, keyFileData.data.size()));
+        std::memcpy(m_key.data(), keyFileData.data.data(), std::min(SHA256_SIZE, keyFileData.data.size()));
         ok = true;
     }
 
-    sodium_memzero(keyFileData.data.data(), static_cast<std::size_t>(keyFileData.data.capacity()));
+    Botan::secure_scrub_memory(keyFileData.data.data(), static_cast<std::size_t>(keyFileData.data.capacity()));
 
     return ok;
 }
@@ -368,13 +384,12 @@ bool FileKey::loadBinary(QIODevice* device)
         return false;
     }
 
-    QByteArray data;
-    if (!Tools::readAllFromDevice(device, data) || data.size() != 32) {
+    Botan::secure_vector<char> data(32);
+    if (device->read(data.data(), 32) != 32 || !device->atEnd()) {
         return false;
     }
 
-    std::memcpy(m_key, data.data(), std::min(SHA256_SIZE, data.size()));
-    sodium_memzero(data.data(), static_cast<std::size_t>(data.capacity()));
+    m_key = data;
     m_type = FixedBinary;
     return true;
 }
@@ -401,15 +416,13 @@ bool FileKey::loadHex(QIODevice* device)
         return false;
     }
 
-    QByteArray key = QByteArray::fromHex(data);
-    sodium_memzero(data.data(), static_cast<std::size_t>(data.capacity()));
-
-    if (key.size() != 32) {
+    data = QByteArray::fromHex(data);
+    if (data.size() != 32) {
         return false;
     }
 
-    std::memcpy(m_key, key.data(), std::min(SHA256_SIZE, key.size()));
-    sodium_memzero(key.data(), static_cast<std::size_t>(key.capacity()));
+    std::memcpy(m_key.data(), data.data(), std::min(SHA256_SIZE, data.size()));
+    Botan::secure_scrub_memory(data.data(), static_cast<std::size_t>(data.capacity()));
 
     m_type = FixedBinaryHex;
     return true;
@@ -433,9 +446,9 @@ bool FileKey::loadHashed(QIODevice* device)
         cryptoHash.addData(buffer);
     } while (!buffer.isEmpty());
 
-    auto result = cryptoHash.result();
-    std::memcpy(m_key, result.data(), std::min(SHA256_SIZE, result.size()));
-    sodium_memzero(result.data(), static_cast<std::size_t>(result.capacity()));
+    buffer = cryptoHash.result();
+    std::memcpy(m_key.data(), buffer.data(), std::min(SHA256_SIZE, buffer.size()));
+    Botan::secure_scrub_memory(buffer.data(), static_cast<std::size_t>(buffer.capacity()));
 
     m_type = Hashed;
     return true;
