@@ -47,17 +47,11 @@ void YubiKey::findValidKeysAsync()
 {
 }
 
-QList<YubiKeySlot> YubiKey::foundKeys()
+YubiKey::KeyMap YubiKey::foundKeys()
 {
     return {};
 }
 
-QString YubiKey::getDisplayName(YubiKeySlot slot)
-{
-    Q_UNUSED(slot);
-    return {};
-}
-
 QString YubiKey::errorMessage()
 {
     return {};
