@@ -1,14 +1,7 @@
 #ifndef KEEPASSX_TOUCHID_H
 #define KEEPASSX_TOUCHID_H
 
-#define TOUCHID_UNDEFINED -1
-#define TOUCHID_AVAILABLE 1
-#define TOUCHID_NOT_AVAILABLE 0
-
-#include <QByteArray>
 #include <QHash>
-#include <QSharedPointer>
-#include <QString>
 
 class TouchID
 {
@@ -18,28 +11,29 @@ public:
 private:
     TouchID()
     {
+       // Nothing to do here
     }
 
-    // TouchID(TouchID const&); // Don't Implement
-    // void operator=(TouchID const&); // Don't implement
-
-    QHash<QString, QByteArray> m_encryptedMasterKeys;
-    int m_available = TOUCHID_UNDEFINED;
-
 public:
     TouchID(TouchID const&) = delete;
-
     void operator=(TouchID const&) = delete;
 
     bool storeKey(const QString& databasePath, const QByteArray& passwordKey);
-
-    QSharedPointer<QByteArray> getKey(const QString& databasePath) const;
+    bool getKey(const QString& databasePath, QByteArray& passwordKey) const;
+    bool containsKey(const QString& databasePath) const;
+    void reset(const QString& databasePath = "");
 
     bool isAvailable();
 
-    bool authenticate(const QString& message = "") const;
+private:
+    static bool isWatchAvailable();
+    static bool isTouchIdAvailable();
 
-    void reset(const QString& databasePath = "");
+    static void deleteKeyEntry(const QString& accountName);
+    static QString databaseKeyName(const QString& databasePath);
+
+private:
+    QHash<QString, QByteArray> m_encryptedMasterKeys;
 };
 
 #endif // KEEPASSX_TOUCHID_H
