@@ -38,7 +38,8 @@ private slots:
     void testResolveNonIdPlaceholdersToUuid();
     void testResolveClonedEntry();
     void testIsRecycled();
-    void testMove();
+    void testMoveUpDown();
+    void testPreviousParentGroup();
 };
 
 #endif // KEEPASSX_TESTENTRY_H
